'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = reactElementToJSXString;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _collapseWhiteSpace = require('collapse-white-space');

var _collapseWhiteSpace2 = _interopRequireDefault(_collapseWhiteSpace);

var _reactAddonsTestUtils = require('react-addons-test-utils');

var _stringifyObject = require('stringify-object');

var _stringifyObject2 = _interopRequireDefault(_stringifyObject);

var _sortobject = require('sortobject');

var _sortobject2 = _interopRequireDefault(_sortobject);

function reactElementToJSXString(ReactElement) {
  return toJSXString({ ReactElement: ReactElement });
}

function toJSXString(_ref) {
  var _ref$ReactElement = _ref.ReactElement;
  var ReactElement = _ref$ReactElement === undefined ? null : _ref$ReactElement;
  var _ref$lvl = _ref.lvl;
  var lvl = _ref$lvl === undefined ? 0 : _ref$lvl;
  var _ref$inline = _ref.inline;
  var inline = _ref$inline === undefined ? false : _ref$inline;

  if (!(0, _reactAddonsTestUtils.isElement)(ReactElement)) {
    throw new Error('react-element-to-jsx-string: Expected a ReactElement');
  }

  var tagName = undefined;

  if (ReactElement.type === undefined) {
    tagName = 'UnknownElement';
  } else {
    tagName = ReactElement.type.name || ReactElement.type.displayName || ReactElement.type;
  }

  var out = '<' + tagName;
  var props = formatProps(ReactElement.props);
  var children = ReactElement.props.children;

  props.forEach(function (prop) {
    if (props.length === 1 || inline) {
      out += ' ';
    } else {
      out += '\n' + spacer(lvl + 1);
    }

    out += prop.name + '=' + prop.value;
  });

  if (props.length > 1 && !inline) {
    out += '\n' + spacer(lvl);
  }

  if (_react2['default'].Children.count(children) > 0) {
    out += '>';
    lvl++;
    if (!inline) {
      out += '\n';
      out += spacer(lvl);
    }

    if (typeof children === 'string') {
      out += children;
    } else {
      out += _react2['default'].Children.toArray(children).map(recurse({ lvl: lvl, inline: inline }));
    }
    if (!inline) {
      out += '\n';
      out += spacer(lvl - 1);
    }
    out += '</' + tagName + '>';
  } else {
    if (props.length <= 1) {
      out += ' ';
    }

    out += '/>';
  }

  return out;
}

function formatProps(props) {
  return Object.keys(props).filter(noChildren).sort().map(function (propName) {
    return {
      name: propName,
      value: formatPropValue(props[propName])
    };
  });
}

function formatPropValue(propValue) {
  if (typeof propValue === 'string') {
    return '"' + propValue + '"';
  }

  var out = '{';

  if (typeof propValue === 'function') {
    out += 'function () {code;}';
  } else if (typeof propValue === 'number') {
    out += propValue;
  } else if ((0, _reactAddonsTestUtils.isElement)(propValue)) {
    out += toJSXString({ ReactElement: propValue, inline: true });
  } else {
    out += stringifyObject(propValue);
  }

  out += '}';

  return out;
}

function recurse(_ref2) {
  var lvl = _ref2.lvl;
  var inline = _ref2.inline;

  return function (ReactElement) {
    return toJSXString({ ReactElement: ReactElement, lvl: lvl, inline: inline });
  };
}

function stringifyObject(obj) {
  // sortobject fails on some types, like regex
  if (Object.keys(obj).length > 0) {
    obj = (0, _sortobject2['default'])(obj);
  }

  return (0, _collapseWhiteSpace2['default'])((0, _stringifyObject2['default'])(obj)).replace(/{ /g, '{').replace(/ }/g, '}').replace(/\[ /g, '[').replace(/ \]/g, ']');
}

function spacer(times) {
  return new Array(times).fill('  ').join('');
}

function noChildren(propName) {
  return propName !== 'children';
}
module.exports = exports['default'];
